@@ -15,6 +15,7 @@ import glob
 from typing import Union, List, Iterable
 
 from obspy import Stream, UTCDateTime, Inventory, Trace
+from obsplus import WaveBank
 from matplotlib.figure import Figure
 from multiprocessing import Process, Lock
 from eqcorrscan import Tribe, Template, Party, Detection
@@ -50,6 +51,9 @@ class RealTimeTribe(Tribe):
         Whether to generate the real-time bokeh plot
     plot_options
         Plotting options parsed to `rt_eqcorrscan.plotting.plot_buffer`
+    wavebank
+        WaveBank to save data to. Used for backfilling by RealTimeTribe.
+        Set to `None` to not use a WaveBank.
     
     sleep_step
         Default sleep-step in seconds while waiting for data. Defaults to 1.0
@@ -76,6 +80,10 @@ class RealTimeTribe(Tribe):
     _min_run_length = 24 * 3600  # Minimum run-length in seconds.
     # Usurped by max_run_length, used to set a threshold for rate calculation.
 
+    # WaveBank management
+    wavebank_lock = Lock()
+    has_wavebank = False
+
     def __init__(
         self,
         name: str = None,
@@ -85,6 +93,7 @@ class RealTimeTribe(Tribe):
         detect_interval: float = 60.,
         plot: bool = True,
         plot_options: dict = None,
+        wavebank: Union[str, WaveBank] = WaveBank("Streaming_WaveBank"),
     ) -> None:
         super().__init__(templates=tribe.templates)
         self.rt_client = rt_client
@@ -104,6 +113,14 @@ class RealTimeTribe(Tribe):
                 if key != "plot_length"})
         self.detections = []
 
+        # Wavebank status to avoid accessing the underlying, lockable, wavebank
+        if isinstance(wavebank, str):
+            wavebank = WaveBank(wavebank)
+        self.__wavebank = wavebank
+        if wavebank:
+            self.has_wavebank = True
+        self._wavebank_warned = False  # Reduce duplicate warnings
+
     def __repr__(self):
         """
         Print information about the tribe.
@@ -167,6 +184,18 @@ class RealTimeTribe(Tribe):
         """
         return {t.name for t in self.templates}
 
+    @property
+    def wavebank(self):
+        return self.__wavebank
+
+    @wavebank.setter
+    def wavebank(self, wavebank: WaveBank):
+        self.__wavebank = wavebank
+        if wavebank:
+            self.has_wavebank = True
+        else:
+            self.has_wavebank = False
+
     def _ensure_templates_have_enough_stations(self, min_stations):
         """ Remove templates that don't have enough stations. """
         self.templates = [
@@ -192,6 +221,67 @@ class RealTimeTribe(Tribe):
                 active_backfillers.append(backfill_process)
         self._backfillers = active_backfillers
 
+    def _access_wavebank(
+        self,
+        method: str,
+        timeout: float = None,
+        *args, **kwargs
+    ):
+        """
+        Thread and process safe access to the wavebank.
+
+        Multiple processes cannot access the underlying HDF5 file at the same
+        time.  This method waits until access to the HDF5 file is available and
+
+        Parameters
+        ----------
+        method
+            Method of wavebank to call
+        timeout
+            Maximum time to try to get access to the file
+        args
+            Arguments passed to method
+        kwargs
+            Keyword arguments passed to method
+
+        Returns
+        -------
+        Whatever should be returned by the method.
+        """
+        if not self.has_wavebank:
+            if not self._wavebank_warned:
+                Logger.error("No wavebank attached to streamer")
+            return None
+        timer, wait_step = 0.0, 0.5
+        with self.wavebank_lock:
+            try:
+                func = self.wavebank.__getattribute__(method)
+            except AttributeError:
+                Logger.error(f"No wavebank method named {method}")
+                return None
+            # Attempt to access the underlying wavebank
+            out = None
+            while timer < timeout:
+                tic = time.time()
+                try:
+                    out = func(*args, **kwargs)
+                    break
+                except (IOError, OSError) as e:
+                    time.sleep(wait_step)
+                toc = time.time()
+                timer += toc - tic
+            else:
+                Logger.error(
+                    f"Waited {timer} s and could not access the wavebank "
+                    f"due to {e}")
+        return out
+
+    def get_wavebank_stream(self, bulk: List[tuple]) -> Stream:
+        """ processsafe get-waveforms-bulk call """
+        st = self._access_wavebank(
+            method="get_waveforms_bulk", timeout=120., bulk=bulk)
+        return st
+
     def _handle_detections(
         self,
         new_party: Party,
@@ -319,6 +409,33 @@ class RealTimeTribe(Tribe):
             pass
         return
 
+    def _start_streaming(self):
+        if not self.rt_client.started:
+            self.rt_client.start()
+        if self.rt_client.can_add_streams:
+            for tr_id in self.expected_seed_ids:
+                self.rt_client.select_stream(
+                    net=tr_id.split('.')[0], station=tr_id.split('.')[1],
+                    selector=tr_id.split('.')[3])
+        else:
+            Logger.warning("Client already in streaming mode,"
+                           " cannot add channels")
+        if not self.rt_client.streaming:
+            self.rt_client.background_run()
+            Logger.info("Started real-time streaming")
+        else:
+            Logger.info("Real-time streaming already running")
+
+    def _runtime_check(self, run_start, max_run_length):
+        run_time = UTCDateTime.now() - run_start
+        Logger.info(
+            f"Run time: {run_time:.2f}s, max_run_length: {max_run_length:.2f}s")
+        if max_run_length and run_time > max_run_length:
+            Logger.critical("Hit maximum run time, stopping.")
+            self.stop()
+            return False
+        return True
+
     def run(
         self,
         threshold: float,
@@ -387,6 +504,9 @@ class RealTimeTribe(Tribe):
         The party created - will not contain detections expired by
         `keep_detections` threshold.
         """
+        restart_interval = 600.0
+        # Squash duplicate channels to avoid excessive channels
+        self.templates = [squash_duplicates(t) for t in self.templates]
         # Reshape the templates first
         if len(self.templates) > 0:
             self.templates = reshape_templates(
@@ -423,21 +543,11 @@ class RealTimeTribe(Tribe):
         detect_directory = detect_directory.format(name=self.name)
         if not os.path.isdir(detect_directory):
             os.makedirs(detect_directory)
-        if not self.rt_client.started:
-            self.rt_client.start()
-        if self.rt_client.can_add_streams:
-            for tr_id in self.expected_seed_ids:
-                self.rt_client.select_stream(
-                    net=tr_id.split('.')[0], station=tr_id.split('.')[1],
-                    selector=tr_id.split('.')[3])
-        else:
-            Logger.warning("Client already in streaming mode,"
-                           " cannot add channels")
-        if not self.rt_client.busy:
-            self.rt_client.background_run()
-            Logger.info("Started real-time streaming")
-        else:
-            Logger.info("Real-time streaming already running")
+        # Get this locally before streaming starts
+        buffer_capacity = self.rt_client.buffer_capacity  
+        # Start the streamer
+        self._start_streaming()
+
         Logger.info("Detection will use the following data: {0}".format(
             self.expected_seed_ids))
         if backfill_client and backfill_to:
@@ -450,9 +560,9 @@ class RealTimeTribe(Tribe):
                 except IndexError:
                     continue
                 endtime = tr_in_buffer.stats.starttime
-                if endtime - backfill_to > self.rt_client.buffer_capacity:
+                if endtime - backfill_to > buffer_capacity:
                     Logger.info("Truncating backfill to buffer length")
-                    starttime = endtime - self.rt_client.buffer_capacity
+                    starttime = endtime - buffer_capacity
                 else:
                     starttime = backfill_to
                 try:
@@ -490,111 +600,161 @@ class RealTimeTribe(Tribe):
             plot_detections=plot_detections, save_waveforms=save_waveforms,
             maximum_backfill=first_data, endtime=None,
             min_stations=min_stations)
+
         try:
             while self.busy:
-                self._running = True  # Lock tribe
-                start_time = UTCDateTime.now()
-                st = self.rt_client.stream.split().merge()
-                # Split to remove trailing mask
-                if len(st) == 0:
-                    Logger.warning("No data")
-                    continue
-                # Cope with data that doesn't come
-                last_data = max(tr.stats.endtime for tr in st)
-                # Remove any data that shouldn't be there - sometimes GeoNet's
-                # Seedlink client gives old data.
-                st.trim(
-                    starttime=last_data - (self.rt_client.buffer_capacity + 20.0),
-                    endtime=last_data)
-                if detection_iteration > 0:
-                    # For the first run we want to detect in everything we have.
-                    st.trim(
-                        starttime=last_data - self.minimum_data_for_detection,
-                        endtime=last_data)
-                if len(st) == 0:
-                    Logger.warning("No data")
-                    continue
-                # Remove short channels
-                st.traces = [
-                    tr for tr in st
-                    if _numpy_len(tr.data) >= (
-                            .8 * self.minimum_data_for_detection)]
-                Logger.info("Starting detection run")
-                Logger.info("Using data: \n{0}".format(
-                    st.__str__(extended=True)))
                 try:
-                    Logger.debug("Currently have {0} templates in tribe".format(
-                        len(self)))
-                    new_party = self.detect(
-                        stream=st, plot=False, threshold=threshold,
-                        threshold_type=threshold_type, trig_int=trig_int,
-                        xcorr_func="fftw", concurrency="concurrent",
-                        cores=self.max_correlation_cores,
-                        process_cores=self.process_cores,
-                        parallel_process=self._parallel_processing,
-                        ignore_bad_data=True, **kwargs)
-                    Logger.info("Completed detection")
-                except Exception as e:  # pragma: no cover
-                    Logger.error(e)
-                    Logger.error(traceback.format_exc())
-                    if "Cannot allocate memory" in str(e):
-                        Logger.error("Out of memory, stopping this detector")
-                        self.stop()
-                        break
+                    self._running = True  # Lock tribe
+                    start_time = UTCDateTime.now()
+                    st = self.rt_client.stream.split().merge()
+                    if self.has_wavebank:
+                        st = _check_stream_is_int(st)
+                        try:
+                            self._access_wavebank(
+                                method="put_waveforms", timeout=120.,
+                                stream=st)
+                        except Exception as e:
+                            Logger.error(
+                                f"Could not write to wavebank due to {e}")
+                    last_data_received = self.rt_client.last_data
+                    # Split to remove trailing mask
+                    if len(st) == 0:
+                        Logger.warning("No data")
+                        continue
                     Logger.info(
-                        "Waiting for {0:.2f}s and hoping this gets "
-                        "better".format(self.detect_interval))
-                    time.sleep(self.detect_interval)
-                    continue
-                Logger.info(f"Trying to get lock - lock status: {self.lock}")
-                with self.lock:
-                    if len(new_party) > 0:
-                        self._handle_detections(
-                            new_party, trig_int=trig_int,
-                            hypocentral_separation=hypocentral_separation,
-                            endtime=last_data - keep_detections,
-                            detect_directory=detect_directory,
-                            save_waveforms=save_waveforms,
-                            plot_detections=plot_detections, st=st)
-                    self._remove_old_detections(last_data - keep_detections)
-                    Logger.info("Party now contains {0} detections".format(
-                        len(self.detections)))
-                self._running = False  # Release lock
-                run_time = UTCDateTime.now() - start_time
-                Logger.info("Detection took {0:.2f}s".format(run_time))
-                if self.detect_interval <= run_time:
-                    Logger.warning(
-                        "detect_interval {0:.2f} shorter than run-time "
-                        "{1:.2f}, increasing detect_interval to {2:.2f}".format(
-                            self.detect_interval, run_time, run_time + 10))
-                    self.detect_interval = run_time + 10
-                Logger.debug("This step took {0:.2f}s total".format(run_time))
-                Logger.info("Waiting {0:.2f}s until next run".format(
-                    self.detect_interval - run_time))
-                detection_iteration += 1
-                self._wait(
-                    wait=(self.detect_interval - run_time) / self._speed_up,
-                    detection_kwargs=detection_kwargs)
-                if max_run_length and UTCDateTime.now() > run_start + max_run_length:
-                    Logger.critical("Hit maximum run time, stopping.")
-                    self.stop()
-                    break
-                if minimum_rate and UTCDateTime.now() > run_start + self._min_run_length:
-                    _rate = average_rate(
-                        self.detections,
-                        starttime=max(last_data - keep_detections, first_data),
-                        endtime=last_data)
-                    if _rate < minimum_rate:
-                        Logger.critical(
-                            "Rate ({0:.2f}) has dropped below minimum rate, "
-                            "stopping.".format(_rate))
-                        self.stop()
+                        f"Streaming Client last received data at "
+                        f"{self.rt_client.last_data}")
+                    stream_end = max(tr.stats.endtime for tr in st)
+                    min_stream_end = min(tr.stats.endtime for tr in st)
+                    Logger.info(
+                        f"Real-time client provided data: \n{st.__str__(extended=True)}")
+                    # Cope with data that doesn't come
+                    if start_time - last_data_received > restart_interval:
+                        Logger.warning(
+                            "The streaming client has not given any new data for "
+                            f"{restart_interval} seconds. Restarting Streaming client")
+                        Logger.info(f"start_time: {start_time}, last_data_received: {last_data_received}, stream_end: {stream_end}")
+                        Logger.info("Stopping streamer")
+                        self.rt_client.background_stop()
+                        self.rt_client.stop()
+                        # Get a clean instance just in case
+                        Logger.info("Starting streamer")
+                        self._start_streaming()
+                        Logger.info("Streamer started")
+                        st = self.rt_client.stream.split().merge()  # Get data again.
+                    Logger.info("Streaming client seems healthy")
+                    # Remove any data that shouldn't be there - sometimes GeoNet's
+                    # Seedlink client gives old data.
+                    Logger.info(
+                        f"Trimming between {stream_end - (buffer_capacity + 20.0)} "
+                        f"and {stream_end}")
+                    st.trim(
+                        starttime=stream_end - (buffer_capacity + 20.0),
+                        endtime=stream_end)
+                    if detection_iteration > 0:
+                        # For the first run we want to detect in everything we have.
+                        # Otherwise trim so that all channels have at-least minimum data for detection
+                        st.trim(
+                            starttime=min_stream_end - self.minimum_data_for_detection,
+                            endtime=stream_end)
+                    Logger.info("Trimmed data")
+                    if len(st) == 0:
+                        Logger.warning("No data")
+                        continue
+                    # Remove short channels
+                    st.traces = [
+                        tr for tr in st
+                        if _numpy_len(tr.data) >= (
+                                .8 * self.minimum_data_for_detection)]
+                    Logger.info("Starting detection run")
+                    Logger.info("Using data: \n{0}".format(
+                        st.__str__(extended=True)))
+                    try:
+                        Logger.debug("Currently have {0} templates in tribe".format(
+                            len(self)))
+                        new_party = self.detect(
+                            stream=st, plot=False, threshold=threshold,
+                            threshold_type=threshold_type, trig_int=trig_int,
+                            xcorr_func="fftw", concurrency="concurrent",
+                            cores=self.max_correlation_cores,
+                            process_cores=self.process_cores,
+                            parallel_process=self._parallel_processing,
+                            ignore_bad_data=True, copy_data=False,
+                            **kwargs)
+                        Logger.info("Completed detection")
+                    except Exception as e:  # pragma: no cover
+                        Logger.error(e)
+                        Logger.error(traceback.format_exc())
+                        if "Cannot allocate memory" in str(e):
+                            Logger.error("Out of memory, stopping this detector")
+                            self.stop()
+                            break
+                        if not self._runtime_check(
+                                run_start=run_start, max_run_length=max_run_length):
+                            break
+                        Logger.info(
+                            "Waiting for {0:.2f}s and hoping this gets "
+                            "better".format(self.detect_interval))
+                        time.sleep(self.detect_interval)
+                        continue
+                    Logger.info(f"Trying to get lock - lock status: {self.lock}")
+                    with self.lock:
+                        if len(new_party) > 0:
+                            self._handle_detections(
+                                new_party, trig_int=trig_int,
+                                hypocentral_separation=hypocentral_separation,
+                                endtime=stream_end - keep_detections,
+                                detect_directory=detect_directory,
+                                save_waveforms=save_waveforms,
+                                plot_detections=plot_detections, st=st)
+                        self._remove_old_detections(
+                            stream_end - keep_detections)
+                        Logger.info("Party now contains {0} detections".format(
+                            len(self.detections)))
+                    self._running = False  # Release lock
+                    run_time = UTCDateTime.now() - start_time
+                    Logger.info("Detection took {0:.2f}s".format(run_time))
+                    if self.detect_interval <= run_time:
+                        Logger.warning(
+                            "detect_interval {0:.2f} shorter than run-time "
+                            "{1:.2f}, increasing detect_interval to {2:.2f}".format(
+                                self.detect_interval, run_time, run_time + 10))
+                        self.detect_interval = run_time + 10
+                    Logger.info("Iteration {0} took {1:.2f}s total".format(
+                        detection_iteration, run_time))
+                    Logger.info("Waiting {0:.2f}s until next run".format(
+                        self.detect_interval - run_time))
+                    detection_iteration += 1
+                    self._wait(
+                        wait=(self.detect_interval - run_time) / self._speed_up,
+                        detection_kwargs=detection_kwargs)
+                    if not self._runtime_check(
+                            run_start=run_start, max_run_length=max_run_length):
+                        break
+                    if minimum_rate and UTCDateTime.now() > run_start + self._min_run_length:
+                        _rate = average_rate(
+                            self.detections,
+                            starttime=max(
+                                stream_end - keep_detections, first_data),
+                            endtime=stream_end)
+                        if _rate < minimum_rate:
+                            Logger.critical(
+                                "Rate ({0:.2f}) has dropped below minimum rate, "
+                                "stopping.".format(_rate))
+                            self.stop()
+                            break
+                    gc.collect()
+                    # Memory output
+                    # sum1 = summary.summarize(muppy.get_objects())
+                    # summary.print_(sum1)
+                except Exception as e:
+                    Logger.critical(f"Uncaught error: {e}")
+                    Logger.error(traceback.format_exc())
+                    if not self._runtime_check(
+                            run_start=run_start, max_run_length=max_run_length):
                         break
-                gc.collect()
-                # Memory output
-                # sum1 = summary.summarize(muppy.get_objects())
-                # summary.print_(sum1)
         finally:
+            Logger.critical("Stopping")
             self.stop()
         return self.party
 
@@ -707,6 +867,8 @@ class RealTimeTribe(Tribe):
             new_tribe = Tribe(templates)
         else:
             new_tribe = templates
+        # Squash duplicate channels to avoid excessive channels
+        new_tribe.templates = [squash_duplicates(t) for t in new_tribe.templates]
         # Reshape
         new_tribe.templates = reshape_templates(
             templates=new_tribe.templates, used_seed_ids=self.expected_seed_ids)
@@ -784,38 +946,8 @@ class RealTimeTribe(Tribe):
         endtime
             Time to stop the backfill, if None will run to now.
         """
-        self._number_of_backfillers += 1
-        backfill_process = Process(
-            target=self._backfill,
-            args=(templates, threshold, threshold_type, trig_int,
-                  hypocentral_separation, keep_detections, detect_directory,
-                  plot_detections, save_waveforms, maximum_backfill, endtime),
-            kwargs=kwargs, name=f"Backfiller_{self._number_of_backfillers}")
-        backfill_process.start()
-        self._backfillers.append(backfill_process)
-
-    def _backfill(
-        self,
-        templates: Union[List[Template], Tribe],
-        threshold: float,
-        threshold_type: str,
-        trig_int: float,
-        hypocentral_separation: float = None,
-        keep_detections: float = 86400,
-        detect_directory: str = "{name}/detections",
-        plot_detections: bool = True,
-        save_waveforms: bool = True,
-        maximum_backfill: float = None,
-        endtime: UTCDateTime = None,
-        **kwargs
-    ) -> None:
-        """ Background backfill method """
-        if isinstance(templates, Tribe):
-            new_tribe = templates
-        else:
-            new_tribe = Tribe(templates)
-        Logger.info(f"Backfilling with {len(new_tribe)} templates")
-        # Get the stream
+        # Get the stream - Only let the main process get the Stream
+        Logger.info("Acquiring stream from wavebank")
         endtime = endtime or UTCDateTime.now()
         if maximum_backfill is not None:
             if isinstance(maximum_backfill, (float, int)):
@@ -830,7 +962,7 @@ class RealTimeTribe(Tribe):
         else:
             starttime = UTCDateTime(0)
         Logger.info(f"Backfilling between {starttime} and {endtime}")
-        if starttime >= endtime or not self.rt_client.has_wavebank:
+        if starttime >= endtime or not self.has_wavebank:
             return
         if self.expected_seed_ids and len(self.expected_seed_ids) > 0:
             bulk = []
@@ -845,31 +977,91 @@ class RealTimeTribe(Tribe):
             Logger.warning("No bulk")
             return
         Logger.info(f"Getting stations for backfill: {bulk}")
-        st = self.rt_client.get_wavebank_stream(bulk)
+        st = self.get_wavebank_stream(bulk)
+        
+        self._number_of_backfillers += 1
+
+        backfill_process = Process(
+            target=self._backfill,
+            args=(templates, st, threshold, threshold_type, trig_int,
+                  hypocentral_separation, keep_detections, detect_directory,
+                  plot_detections, save_waveforms, endtime),
+            kwargs=kwargs, name=f"Backfiller_{self._number_of_backfillers}")
+        backfill_process.start()
+        self._backfillers.append(backfill_process)
+        Logger.info("Backfill process started, returning")
+        return
+
+    def _backfill(
+        self,
+        templates: Union[List[Template], Tribe],
+        st: Stream,
+        threshold: float,
+        threshold_type: str,
+        trig_int: float,
+        hypocentral_separation: float = None,
+        keep_detections: float = 86400,
+        detect_directory: str = "{name}/detections",
+        plot_detections: bool = True,
+        save_waveforms: bool = True,
+        endtime: UTCDateTime = None,
+        **kwargs
+    ) -> None:
+        """ Background backfill method """
+        if isinstance(templates, Tribe):
+            new_tribe = templates
+        else:
+            new_tribe = Tribe(templates)
+        # Squash duplicate channels to avoid excessive channels
+        new_tribe.templates = [squash_duplicates(t) for t in new_tribe.templates]
+        # Reshape
+        new_tribe.templates = reshape_templates(
+            templates=new_tribe.templates, used_seed_ids=self.expected_seed_ids)
+        Logger.info(f"Backfilling with {len(new_tribe)} templates")
         Logger.debug("Additional templates to be run: \n{0} "
                      "templates".format(len(new_tribe)))
 
         Logger.info("Starting backfill detection run with:")
         Logger.info(st.__str__(extended=True))
-        new_party = new_tribe.detect(
-            stream=st, plot=False, threshold=threshold,
-            threshold_type=threshold_type, trig_int=trig_int,
-            xcorr_func="fftw", concurrency="concurrent",
-            cores=self.max_correlation_cores,
-            parallel_process=self._parallel_processing,
-            process_cores=self.process_cores, **kwargs)
-        detect_directory = detect_directory.format(name=self.name)
-        Logger.info("Backfill detection completed - handling detections")
-        if len(new_party) > 0:
-            Logger.info(f"Lock status: {self.lock}")
-            with self.lock:  # The only time the state is altered
+        # Break into chunks so that detections can be handled as they happen
+        starttime, endtime = (
+            min(tr.stats.starttime for tr in st),
+            max(tr.stats.endtime for tr in st))
+        # Send off twice the minimum data to allow for overlaps.
+        _starttime, _endtime = (
+            starttime, starttime + 2 * self.minimum_data_for_detection)
+        while _endtime < endtime + self.minimum_data_for_detection:
+            st_chunk = st.slice(_starttime, _endtime)
+            try:
+                new_party = new_tribe.detect(
+                    stream=st_chunk, plot=False, threshold=threshold,
+                    threshold_type=threshold_type, trig_int=trig_int,
+                    xcorr_func="fftw", concurrency="concurrent",
+                    cores=self.max_correlation_cores,
+                    parallel_process=self._parallel_processing,
+                    process_cores=self.process_cores, copy_data=False,
+                    **kwargs)
+            except Exception as e:
+                Logger.error(e)
+                _starttime += self.minimum_data_for_detection
+                _endtime += self.minimum_data_for_detection
+                continue
+            detect_directory = detect_directory.format(name=self.name)
+            Logger.info(
+                f"Backfill detection between {_starttime} and {_endtime} "
+                f"completed - handling detections")
+            if len(new_party) > 0:
                 Logger.info(f"Lock status: {self.lock}")
-                self._handle_detections(
-                    new_party=new_party, detect_directory=detect_directory,
-                    endtime=endtime - keep_detections,
-                    plot_detections=plot_detections,
-                    save_waveforms=save_waveforms, st=st, trig_int=trig_int,
-                    hypocentral_separation=hypocentral_separation)
+                with self.lock:  # The only time the state is altered
+                    Logger.info(f"Lock status: {self.lock}")
+                    self._handle_detections(
+                        new_party=new_party, detect_directory=detect_directory,
+                        endtime=endtime - keep_detections,
+                        plot_detections=plot_detections,
+                        save_waveforms=save_waveforms, st=st, trig_int=trig_int,
+                        hypocentral_separation=hypocentral_separation)
+            _starttime += self.minimum_data_for_detection
+            _endtime += self.minimum_data_for_detection
         return
 
     def stop(self, write_stopfile: bool = False) -> None:
@@ -943,6 +1135,58 @@ def reshape_templates(
     return templates_back
 
 
+def squash_duplicates(template: Template):
+    """
+    Remove duplicate channels in templates. 
+
+    This happens when there are duplicate picks, and it fucks shit up.
+    More explicitly (less?): when there are duplicate picks, duplicate
+    channels appear in the template, which are then extended to all
+    templates meaning that many copies of a template are run, resulting
+    in excessive detection bias on one channel, and very expensive templates
+    without good reason.
+    """
+    from collections import Counter
+    # Check if there are duplicate channels first
+    seed_ids = Counter(tr.id for tr in template.st)
+    if seed_ids.most_common(1)[0][1] == 1:
+        # Do nothing, no duplicates
+        return template
+    unique_template_st = Stream()
+    unique_event_picks = []
+    for seed_id, repeats in seed_ids.most_common():
+        seed_id_picks = [p for p in template.event.picks 
+                         if p.waveform_id.get_seed_string() == seed_id 
+                         and p.phase_hint[0] in "PS"]
+        if repeats == 1:
+            unique_template_st += template.st.select(id=seed_id)
+            unique_event_picks.append(seed_id_picks[0])
+            continue
+        # Now we get to doing something - get the stream and the picks
+        repeat_stream = template.st.select(id=seed_id)
+        # Get the unique start-times
+        unique_traces = {tr.stats.starttime.datetime: tr 
+                         for tr in repeat_stream.traces}
+        if len(unique_traces) == 1:
+            unique_template_st += unique_traces.popitem()[1]
+            unique_event_picks.append(seed_id_picks[0])
+            continue
+        # If there are more than one unique start-time then we need to 
+        # find the appropriate pick for each
+        unique_picks = {f"{p.phase_hint}_{p.time}": p for p in seed_id_picks}
+        for pick in unique_picks.values():
+            expected_starttime = pick.time - template.prepick
+            tr = unique_traces.get(expected_starttime.datetime, None)
+            if tr:
+                unique_event_picks.append(pick)
+                unique_template_st += tr
+            else:
+                Logger.debug(f"No trace for pick at {pick.time}")
+    template.st = unique_template_st
+    template.event.picks = unique_event_picks
+    return template
+
+
 def _write_detection(
     detection: Detection,
     detect_directory: str,
@@ -979,12 +1223,16 @@ def _write_detection(
     from rt_eqcorrscan.plotting.plot_event import plot_event
 
     _path = os.path.join(
-        detect_directory, detection.detect_time.strftime("%Y/%j"))
+        detect_directory, detection.detect_time.strftime("%Y"),
+        detection.detect_time.strftime("%j"))
     if not os.path.isdir(_path):
         os.makedirs(_path)
     _filename = os.path.join(
         _path, detection.detect_time.strftime("%Y%m%dT%H%M%S"))
-    detection.event.write(f"{_filename}.xml", format="QUAKEML")
+    try:
+        detection.event.write(f"{_filename}.xml", format="QUAKEML")
+    except Exception as e:
+        Logger.error(f"Could not write event file due to {e}")
     detection.event.picks.sort(key=lambda p: p.time)
     st = stream.slice(
         detection.event.picks[0].time - 10,
@@ -993,13 +1241,32 @@ def _write_detection(
         # Make plot
         fig = plot_event(fig=fig, event=detection.event, st=st,
                          length=90, show=False)
-        fig.savefig(f"{_filename}.png")
+        try:
+            fig.savefig(f"{_filename}.png")
+        except Exception as e:
+            Logger.error(f"Could not write plot due to {e}")
         fig.clf()
     if save_waveform:
-        st.split().write(f"{_filename}.ms", format="MSEED")
+        st = _check_stream_is_int(st)
+        try:
+            st.write(f"{_filename}.ms", format="MSEED")
+        except Exception as e:
+            Logger.error(f"Could not write stream due to {e}")
     return fig
 
 
+def _check_stream_is_int(st):
+    st = st.split()
+    for tr in st:
+        # Ensure data are int32, see https://github.com/obspy/obspy/issues/2683
+        if tr.data.dtype == numpy.int32 and \
+                tr.data.dtype.type != numpy.int32:
+            tr.data = tr.data.astype(numpy.int32, subok=False)
+        if tr.data.dtype.type == numpy.intc:
+            tr.data = tr.data.astype(numpy.int32, subok=False)
+    return st
+
+
 def _numpy_len(arr: Union[numpy.ndarray, numpy.ma.MaskedArray]) -> int:
     """
     Convenience function to return the length of a numpy array.
